#!/usr/bin/env python
# -*- coding: utf-8 -*-
from docutils.nodes import version

# Third party imports

# Project imports
from magma_auth.auth import Auth
from magma_auth.auth import AuthExternal
from magma_auth.utils import validate_token

__version__ = version("magma-auth")
__author__ = "Martanto"
__author_email__ = "martanto@live.com"
__license__ = "MIT"
__copyright__ = "Copyright (c) 2025, MAGMA Indonesia"
__url__ = "https://github.com/martanto/magma-auth"

auth = Auth()
auth_external = AuthExternal()

__all__ = [
    "__version__",
    "__author__",
    "__author_email__",
    "__license__",
    "__copyright__",
    "auth",
    "auth_external",
    "validate_token",
]
